/*============================================================*/
/*============================================================*/
/*==														==*/
/*==					SoundUtils Header					==*/
/*==														==*/
/*============================================================*/
/*============================================================*/


/*========================================================  Prototypes  */

void TurnSMSOn (void);
void TurnSMSOff (void);
void PlaySoundSMS (short);
void PlayIncidentSoundSMS (short);
void FlushSoundQueues (void);
void BeginGamePointSound (void);
void BeginAllTiedSound (void);
void StartCrowdSound (void);
void StartApplauseSound (void);
void Applaud (void);
void StartMobSound (void);
void HandleCrowdSound (void);
void StopCrowdSound (void);
void ResumeCrowdSound (void);
void QueueUpIncidental (short, short, short);
void HandleIncidentalQueue (void);


/*========================================================  External Variables */

extern soundQueue		crowdQueue, incidentalQueue;
extern short			lengthOfApplause, lengthOfMob, whichGame, wasLastSound;
extern short			lastSoundPriority, soundFileRefNum;
extern short			soundPriorities[kMaxNumberOfSounds];
extern Boolean			beamSoundOn, incidentSoundOn, collisionSoundOn, crowdSoundOn;
extern Boolean			smsActivated, soundOn, imTheMaster, gameIsOver, netGameInSession;
extern Boolean			soundLoaded[kMaxNumberOfSounds];
extern Boolean			incidentSoundLoaded[kMaxIncidentalSounds];

